@@ -41,7 +41,21 @@
 #include <unistr.h>		/* u8_to_u32 */
 
 #include "idna.h"		/* _idn2_label_test */
-#include "tr46map.h"		/* defintion for tr46map.c */
+#include "tr46map.h"		/* definition for tr46map.c */
+
+static int set_default_flags(int *flags)
+{
+  if (((*flags) & IDN2_TRANSITIONAL) && ((*flags) & IDN2_NONTRANSITIONAL))
+    return IDN2_INVALID_FLAGS;
+
+  if (((*flags) & (IDN2_TRANSITIONAL|IDN2_NONTRANSITIONAL)) && ((*flags) & IDN2_NO_TR46))
+    return IDN2_INVALID_FLAGS;
+
+  if (!((*flags) & (IDN2_NO_TR46|IDN2_TRANSITIONAL)))
+    *flags |= IDN2_NONTRANSITIONAL;
+
+  return IDN2_OK;
+}
 
 static int
 label (const uint8_t * src, size_t srclen, uint8_t * dst, size_t * dstlen,
@@ -89,7 +103,10 @@ label (const uint8_t * src, size_t srclen, uint8_t * dst, size_t * dstlen,
 	TEST_DISALLOWED |
 	TEST_CONTEXTJ_RULE |
 	TEST_CONTEXTO_WITH_RULE |
-	TEST_UNASSIGNED | TEST_BIDI, p, plen);
+	TEST_UNASSIGNED | TEST_BIDI |
+	((flags & IDN2_NONTRANSITIONAL) ? TEST_NONTRANSITIONAL : 0) |
+	((flags & IDN2_USE_STD3_ASCII_RULES) ? 0 : TEST_ALLOW_STD3_DISALLOWED),
+	p, plen);
 
       if (rc != IDN2_OK)
 	{
@@ -385,7 +402,7 @@ idn2_lookup_u8 (const uint8_t * src, uint8_t ** lookupname, int flags)
   size_t lookupnamelen = 0;
   uint8_t _lookupname[IDN2_DOMAIN_MAX_LENGTH + 1];
   uint8_t _mapped[IDN2_DOMAIN_MAX_LENGTH + 1];
-  int rc, tr46_mode = 0;
+  int rc;
 
   if (src == NULL)
     {
@@ -394,16 +411,11 @@ idn2_lookup_u8 (const uint8_t * src, uint8_t ** lookupname, int flags)
       return IDN2_OK;
     }
 
-  if ((flags & (IDN2_TRANSITIONAL | IDN2_NONTRANSITIONAL)) ==
-      (IDN2_TRANSITIONAL | IDN2_NONTRANSITIONAL))
-    return IDN2_INVALID_FLAGS;
+  rc = set_default_flags(&flags);
+  if (rc != IDN2_OK)
+    return rc;
 
-  if (flags & IDN2_TRANSITIONAL)
-    tr46_mode = IDN2_TRANSITIONAL;
-  else if (flags & IDN2_NONTRANSITIONAL)
-    tr46_mode = IDN2_NONTRANSITIONAL;
-
-  if (tr46_mode)
+  if (!(flags & IDN2_NO_TR46))
     {
       uint8_t *out;
       size_t outlen;
@@ -511,8 +523,11 @@ idn2_lookup_ul (const char * src, char ** lookupname, int flags)
 
   if (src)
     {
-      utf8src = u8_strconv_from_locale (src);
-      if (utf8src == NULL)
+      const char *encoding = locale_charset ();
+
+      utf8src = u8_strconv_from_encoding (src, encoding, iconveh_error);
+
+      if (!utf8src)
 	{
 	  if (errno == ENOMEM)
 	    return IDN2_MALLOC;
@@ -552,8 +567,11 @@ idn2_lookup_ul (const char * src, char ** lookupname, int flags)
  * range to begin with (although it could fail). Applying the ToASCII operation multiple
  * effect as applying it just once.
  *
- * When unsure, it is recommended to call this function with the
- * %IDN2_NONTRANSITIONAL and %IDN2_NFC_INPUT flags.
+ * The default behavior of this function (when flags are zero) is to apply
+ * the IDNA2008 rules without the TR46 amendments. As the TR46
+ * non-transitional processing is nowadays ubiquitous, when unsure, it is
+ * recommended to call this function with the %IDN2_NONTRANSITIONAL
+ * and the %IDN2_NFC_INPUT flags for compatibility with other software.
  *
  * Return value: Returns %IDN2_OK on success, or error code.
  *
@@ -617,8 +635,11 @@ idn2_to_ascii_4i (const uint32_t * input, size_t inlen, char * output, int flags
  * rules.  The domain name may contain several labels, separated by dots.
  * The output buffer must be deallocated by the caller.
  *
- * When unsure, it is recommended to call this function with the
- * %IDN2_NONTRANSITIONAL and %IDN2_NFC_INPUT flags.
+ * The default behavior of this function (when flags are zero) is to apply
+ * the IDNA2008 rules without the TR46 amendments. As the TR46
+ * non-transitional processing is nowadays ubiquitous, when unsure, it is
+ * recommended to call this function with the %IDN2_NONTRANSITIONAL
+ * and the %IDN2_NFC_INPUT flags for compatibility with other software.
  *
  * Return value: Returns %IDN2_OK on success, or error code.
  *
@@ -662,8 +683,11 @@ idn2_to_ascii_4z (const uint32_t * input, char ** output, int flags)
  * rules.  The domain name may contain several labels, separated by dots.
  * The output buffer must be deallocated by the caller.
  *
- * When unsure, it is recommended to call this function with the
- * %IDN2_NONTRANSITIONAL and %IDN2_NFC_INPUT flags.
+ * The default behavior of this function (when flags are zero) is to apply
+ * the IDNA2008 rules without the TR46 amendments. As the TR46
+ * non-transitional processing is nowadays ubiquitous, when unsure, it is
+ * recommended to call this function with the %IDN2_NONTRANSITIONAL
+ * and the %IDN2_NFC_INPUT flags for compatibility with other software.
  *
  * Return value: Returns %IDN2_OK on success, or error code.
  *
@@ -685,8 +709,11 @@ idn2_to_ascii_8z (const char * input, char ** output, int flags)
  * rules.  The domain name may contain several labels, separated by dots.
  * The output buffer must be deallocated by the caller.
  *
- * When unsure, it is recommended to call this function with the
- * %IDN2_NONTRANSITIONAL and %IDN2_NFC_INPUT flags.
+ * The default behavior of this function (when flags are zero) is to apply
+ * the IDNA2008 rules without the TR46 amendments. As the TR46
+ * non-transitional processing is nowadays ubiquitous, when unsure, it is
+ * recommended to call this function with the %IDN2_NONTRANSITIONAL
+ * and the %IDN2_NFC_INPUT flags for compatibility with other software.
  *
  * Returns: %IDN2_OK on success, or error code.
  * Same as described in idn2_lookup_ul() documentation.
